@@ -7,14 +7,12 @@ MODFLOW Guide
 <http://water.usgs.gov/ogw/modflow/MODFLOW-2005-Guide/index.html?evt.htm>`_.
 
 """
-import sys
-
 import numpy as np
-from .mfparbc import ModflowParBc as mfparbc
-from ..utils import Transient2d, Util2d
-from ..utils.utils_def import get_pak_vals_shape
 
 from ..pakbase import Package
+from ..utils import Transient2d, Util2d
+from ..utils.utils_def import get_pak_vals_shape
+from .mfparbc import ModflowParBc as mfparbc
 
 
 class ModflowEvt(Package):
@@ -105,48 +103,27 @@ class ModflowEvt(Package):
             unitnumber = ModflowEvt._defaultunit()
 
         # set filenames
-        if filenames is None:
-            filenames = [None, None]
-        elif isinstance(filenames, str):
-            filenames = [filenames, None]
-        elif isinstance(filenames, list):
-            if len(filenames) < 2:
-                filenames.append(None)
+        filenames = self._prepare_filenames(filenames, 2)
 
         # update external file information with cbc output, if necessary
         if ipakcb is not None:
-            fname = filenames[1]
             model.add_output_file(
-                ipakcb, fname=fname, package=ModflowEvt._ftype()
+                ipakcb, fname=filenames[1], package=self._ftype()
             )
         else:
             ipakcb = 0
 
-        # Fill namefile items
-        name = [ModflowEvt._ftype()]
-        units = [unitnumber]
-        extra = [""]
-
-        # set package name
-        fname = [filenames[0]]
-
-        # Call ancestor's init to set self.parent, extension, name and unit number
-        Package.__init__(
-            self,
+        # call base package constructor
+        super().__init__(
             model,
             extension=extension,
-            name=name,
-            unit_number=units,
-            extra=extra,
-            filenames=fname,
+            name=self._ftype(),
+            unit_number=unitnumber,
+            filenames=filenames[0],
         )
 
         nrow, ncol, nlay, nper = self.parent.nrow_ncol_nlay_nper
-        self.heading = (
-            "# {} package for ".format(self.name[0])
-            + " {}, ".format(model.version_types[model.version])
-            + "generated by Flopy."
-        )
+        self._generate_heading()
         self.url = "evt.htm"
         self.nevtop = nevtop
         self.ipakcb = ipakcb
@@ -203,8 +180,8 @@ class ModflowEvt(Package):
             f_evt = f
         else:
             f_evt = open(self.fn_path, "w")
-        f_evt.write("{0:s}\n".format(self.heading))
-        f_evt.write("{0:10d}{1:10d}\n".format(self.nevtop, self.ipakcb))
+        f_evt.write(f"{self.heading}\n")
+        f_evt.write(f"{self.nevtop:10d}{self.ipakcb:10d}\n")
         if self.nevtop == 2 and not self.parent.structured:
             mxndevt = np.max(
                 [
@@ -212,7 +189,7 @@ class ModflowEvt(Package):
                     for kper, u2d in self.ievt.transient_2ds.items()
                 ]
             )
-            f_evt.write("{0:10d}\n".format(mxndevt))
+            f_evt.write(f"{mxndevt:10d}\n")
 
         for n in range(nper):
             insurf, surf = self.surf.get_kper_entry(n)
@@ -221,13 +198,9 @@ class ModflowEvt(Package):
             inievt, ievt = self.ievt.get_kper_entry(n)
             if self.nevtop == 2 and not self.parent.structured:
                 inievt = self.ievt[n].array.size
-            comment = "Evapotranspiration  dataset 5 for stress period " + str(
-                n + 1
-            )
+            comment = f"Evapotranspiration dataset 5 for stress period {n + 1}"
             f_evt.write(
-                "{0:10d}{1:10d}{2:10d}{3:10d} # {4:s}\n".format(
-                    insurf, inevtr, inexdp, inievt, comment
-                )
+                f"{insurf:10d}{inevtr:10d}{inexdp:10d}{inievt:10d} # {comment}\n"
             )
             if insurf >= 0:
                 f_evt.write(surf)
@@ -275,7 +248,7 @@ class ModflowEvt(Package):
 
         """
         if model.verbose:
-            sys.stdout.write("loading evt package file...\n")
+            print("loading evt package file...")
 
         openfile = not hasattr(f, "read")
         if openfile:
@@ -346,11 +319,7 @@ class ModflowEvt(Package):
 
             if insurf >= 0:
                 if model.verbose:
-                    print(
-                        "   loading surf stress period {0:3d}...".format(
-                            iper + 1
-                        )
-                    )
+                    print(f"   loading surf stress period {iper + 1:3d}...")
                 t = Util2d.load(
                     f, model, u2d_shape, np.float32, "surf", ext_unit_dict
                 )
@@ -361,9 +330,7 @@ class ModflowEvt(Package):
                 if npar == 0:
                     if model.verbose:
                         print(
-                            "   loading evtr stress period {0:3d}...".format(
-                                iper + 1
-                            )
+                            f"   loading evtr stress period {iper + 1:3d}..."
                         )
                     t = Util2d.load(
                         f,
@@ -400,11 +367,7 @@ class ModflowEvt(Package):
             evtr[iper] = current_evtr
             if inexdp >= 0:
                 if model.verbose:
-                    print(
-                        "   loading exdp stress period {0:3d}...".format(
-                            iper + 1
-                        )
-                    )
+                    print(f"   loading exdp stress period {iper + 1:3d}...")
                 t = Util2d.load(
                     f, model, u2d_shape, np.float32, "exdp", ext_unit_dict
                 )
@@ -414,9 +377,7 @@ class ModflowEvt(Package):
                 if inievt >= 0:
                     if model.verbose:
                         print(
-                            "   loading ievt stress period {0:3d}...".format(
-                                iper + 1
-                            )
+                            f"   loading ievt stress period {iper + 1:3d}..."
                         )
                     t = Util2d.load(
                         f, model, u2d_shape, np.int32, "ievt", ext_unit_dict
