@@ -1,6 +1,5 @@
-import sys
-import collections
 import numpy as np
+
 from ..pakbase import Package
 from ..utils.recarray_utils import create_empty_recarray
 
@@ -95,21 +94,12 @@ class ModflowHob(Package):
         unitnumber=None,
         filenames=None,
     ):
-        """
-        Package constructor
-        """
         # set default unit number of one is not specified
         if unitnumber is None:
             unitnumber = ModflowHob._defaultunit()
 
         # set filenames
-        if filenames is None:
-            filenames = [None, None]
-        elif isinstance(filenames, str):
-            filenames = [filenames, None]
-        elif isinstance(filenames, list):
-            if len(filenames) < 2:
-                filenames.append(None)
+        filenames = self._prepare_filenames(filenames, 2)
 
         # set filenames[1] to hobname if filenames[1] is not None
         if filenames[1] is None:
@@ -117,43 +107,27 @@ class ModflowHob(Package):
                 filenames[1] = hobname
 
         if iuhobsv is not None:
-            fname = filenames[1]
             model.add_output_file(
                 iuhobsv,
-                fname=fname,
+                fname=filenames[1],
                 extension="hob.out",
                 binflag=False,
-                package=ModflowHob._ftype(),
+                package=self._ftype(),
             )
         else:
             iuhobsv = 0
 
-        # Fill namefile items
-        name = [ModflowHob._ftype()]
-        units = [unitnumber]
-        extra = [""]
-
-        # set package name
-        fname = [filenames[0]]
-
-        # Call ancestor's init to set self.parent,
-        # extension, name and unit number
-        Package.__init__(
-            self,
+        # call base package constructor
+        super().__init__(
             model,
             extension=extension,
-            name=name,
-            unit_number=units,
-            extra=extra,
-            filenames=fname,
+            name=self._ftype(),
+            unit_number=unitnumber,
+            filenames=filenames[0],
         )
 
         self.url = "hob.htm"
-        self.heading = (
-            "# {} package for ".format(self.name[0])
-            + " {}, ".format(model.version_types[model.version])
-            + "generated by Flopy."
-        )
+        self._generate_heading()
 
         self.iuhobsv = iuhobsv
         self.hobdry = hobdry
@@ -199,8 +173,8 @@ class ModflowHob(Package):
         for idx, obs in enumerate(self.obs_data):
             if not isinstance(obs, HeadObservation):
                 msg += (
-                    "ModflowHob: obs_data entry {} ".format(idx)
-                    + "is not a HeadObservation instance.\n"
+                    f"ModflowHob: obs_data entry {idx} "
+                    "is not a HeadObservation instance.\n"
                 )
                 continue
             self.nh += obs.nobs
@@ -227,20 +201,20 @@ class ModflowHob(Package):
         f = open(self.fn_path, "w")
 
         # write dataset 0
-        f.write("{}\n".format(self.heading))
+        f.write(f"{self.heading}\n")
 
         # write dataset 1
-        f.write("{:10d}".format(self.nh))
-        f.write("{:10d}".format(self.mobs))
-        f.write("{:10d}".format(self.maxm))
-        f.write("{:10d}".format(self.iuhobsv))
-        f.write("{:10.4g}".format(self.hobdry))
+        f.write(f"{self.nh:10d}")
+        f.write(f"{self.mobs:10d}")
+        f.write(f"{self.maxm:10d}")
+        f.write(f"{self.iuhobsv:10d}")
+        f.write(f"{self.hobdry:10.4g}")
         if self.no_print or "NOPRINT" in self.options:
             f.write("{: >10}".format("NOPRINT"))
         f.write("\n")
 
         # write dataset 2
-        f.write("{:10.4g}\n".format(self.tomulth))
+        f.write(f"{self.tomulth:10.4g}\n")
 
         # write datasets 3-6
         for idx, obs in enumerate(self.obs_data):
@@ -248,44 +222,44 @@ class ModflowHob(Package):
             obsname = obs.obsname
             if isinstance(obsname, bytes):
                 obsname = obsname.decode("utf-8")
-            line = "{:12s}   ".format(obsname)
+            line = f"{obsname:12s}   "
             layer = obs.layer
             if layer >= 0:
                 layer += 1
-            line += "{:10d} ".format(layer)
-            line += "{:10d} ".format(obs.row + 1)
-            line += "{:10d} ".format(obs.column + 1)
+            line += f"{layer:10d} "
+            line += f"{obs.row + 1:10d} "
+            line += f"{obs.column + 1:10d} "
             irefsp = obs.irefsp
             if irefsp >= 0:
                 irefsp += 1
-            line += "{:10d} ".format(irefsp)
+            line += f"{irefsp:10d} "
             if obs.nobs == 1:
                 toffset = obs.time_series_data[0]["toffset"]
                 hobs = obs.time_series_data[0]["hobs"]
             else:
                 toffset = 0.0
                 hobs = 0.0
-            line += "{:20} ".format(toffset)
-            line += "{:10.4f} ".format(obs.roff)
-            line += "{:10.4f} ".format(obs.coff)
-            line += "{:10.4f} ".format(hobs)
-            line += "  # DATASET 3 - Observation {}".format(idx + 1)
-            f.write("{}\n".format(line))
+            line += f"{toffset:20} "
+            line += f"{obs.roff:10.4f} "
+            line += f"{obs.coff:10.4f} "
+            line += f"{hobs:10.4f} "
+            line += f"  # DATASET 3 - Observation {idx + 1}"
+            f.write(f"{line}\n")
 
             # dataset 4
             if len(obs.mlay.keys()) > 1:
                 line = ""
                 for key, value in iter(obs.mlay.items()):
-                    line += "{:5d}{:10.4f}".format(key + 1, value)
-                line += "  # DATASET 4 - Observation {}".format(idx + 1)
-                f.write("{}\n".format(line))
+                    line += f"{key + 1:5d}{value:10.4f}"
+                line += f"  # DATASET 4 - Observation {idx + 1}"
+                f.write(f"{line}\n")
 
             # dataset 5
             if irefsp < 0:
-                line = "{:10d}".format(obs.itt)
+                line = f"{obs.itt:10d}"
                 line += 103 * " "
-                line += "  # DATASET 5 - Observation {}".format(idx + 1)
-                f.write("{}\n".format(line))
+                line += f"  # DATASET 5 - Observation {idx + 1}"
+                f.write(f"{line}\n")
 
             # dataset 6:
             if obs.nobs > 1:
@@ -293,15 +267,13 @@ class ModflowHob(Package):
                     obsname = t["obsname"]
                     if isinstance(obsname, bytes):
                         obsname = obsname.decode("utf-8")
-                    line = "{:12s}   ".format(obsname)
-                    line += "{:10d} ".format(t["irefsp"] + 1)
-                    line += "{:20} ".format(t["toffset"])
-                    line += "{:10.4f} ".format(t["hobs"])
+                    line = f"{obsname:12s}   "
+                    line += f"{t['irefsp'] + 1:10d} "
+                    line += f"{t['toffset']:20} "
+                    line += f"{t['hobs']:10.4f} "
                     line += 55 * " "
-                    line += "  # DATASET 6 - " + "Observation {}.{}".format(
-                        idx + 1, jdx + 1
-                    )
-                    f.write("{}\n".format(line))
+                    line += f"  # DATASET 6 - Observation {idx + 1}.{jdx + 1}"
+                    f.write(f"{line}\n")
 
         # close the hob package file
         f.close()
@@ -344,7 +316,7 @@ class ModflowHob(Package):
         """
 
         if model.verbose:
-            sys.stdout.write("loading hob package file...\n")
+            print("loading hob package file...")
 
         openfile = not hasattr(f, "read")
         if openfile:
@@ -376,6 +348,11 @@ class ModflowHob(Package):
 
         # read datasets 3-6
         nobs = 0
+
+        # set to False for 1st call to ensure that totim cache is updated
+        tmax = model.dis.get_final_totim()
+        use_cached_totim = True
+
         while True:
             # read dataset 3
             line = f.readline()
@@ -397,7 +374,7 @@ class ModflowHob(Package):
             else:
                 line = f.readline()
                 t = line.strip().split()
-                mlay = collections.OrderedDict()
+                mlay = {}
                 if len(t) >= abs(layer) * 2:
                     for j in range(0, abs(layer) * 2, 2):
                         k = int(t[j]) - 1
@@ -429,7 +406,7 @@ class ModflowHob(Package):
                 itt = 1
                 irefsp0 -= 1
                 totim = model.dis.get_totim_from_kper_toffset(
-                    irefsp0, toffset * tomulth
+                    irefsp0, toffset * tomulth, use_cached_totim
                 )
                 names = [obsnam]
                 tsd = [totim, hob]
@@ -449,7 +426,7 @@ class ModflowHob(Package):
                     irefsp = int(t[1]) - 1
                     toffset = float(t[2])
                     totim = model.dis.get_totim_from_kper_toffset(
-                        irefsp, toffset * tomulth
+                        irefsp, toffset * tomulth, use_cached_totim
                     )
                     hob = float(t[3])
                     tsd.append([totim, hob])
@@ -467,6 +444,7 @@ class ModflowHob(Package):
                     obsname=obsnam,
                     mlay=mlay,
                     itt=itt,
+                    tmax=tmax,
                     time_series_data=tsd,
                     names=names,
                 )
@@ -510,7 +488,7 @@ class ModflowHob(Package):
         return 39
 
 
-class HeadObservation(object):
+class HeadObservation:
     """
     Create single HeadObservation instance from a time series array. A list of
     HeadObservation instances are passed to the ModflowHob package.
@@ -544,6 +522,9 @@ class HeadObservation(object):
         observations. itt = 1 specified for heads and itt = 2 specified
         if initial value is head and subsequent changes in head. Only
         specified if irefsp is < 0. Default is 1.
+    tmax : float
+        Maximum simulation time calculated using get_final_totim function of
+        ModflowDis. Added to avoid repetitive calls.
     mlay : dictionary of length (abs(irefsp))
         Key represents zero-based layer numbers for multilayer observations and
         value represents the fractional value for each layer of multilayer
@@ -589,6 +570,7 @@ class HeadObservation(object):
         roff=0.0,
         coff=0.0,
         itt=1,
+        tmax=None,
         mlay=None,
         time_series_data=None,
         names=None,
@@ -628,12 +610,13 @@ class HeadObservation(object):
             for key, value in self.mlay.items():
                 tot += value
             if not (np.isclose(tot, 1.0, rtol=0)):
-                msg = (
+                raise ValueError(
                     "sum of dataset 4 proportions must equal 1.0 - "
-                    + "sum of dataset 4 proportions = {tot} for "
-                    + "observation name {obsname}."
-                ).format(tot=tot, obsname=self.obsname)
-                raise ValueError(msg)
+                    "sum of dataset 4 proportions = {tot} for "
+                    "observation name {obsname}.".format(
+                        tot=tot, obsname=self.obsname
+                    )
+                )
 
         # convert passed time_series_data to a numpy array
         if isinstance(time_series_data, list):
@@ -645,7 +628,8 @@ class HeadObservation(object):
             time_series_data = np.reshape(time_series_data, (1, 2))
 
         # find indices of time series data that are valid
-        tmax = model.dis.get_final_totim()
+        if tmax is None:
+            tmax = model.dis.get_final_totim()
         keep_idx = time_series_data[:, 0] <= tmax
         time_series_data = time_series_data[keep_idx, :]
 
@@ -660,31 +644,30 @@ class HeadObservation(object):
             else:
                 names = []
                 for idx in range(self.nobs):
-                    names.append("{}.{}".format(obsname, idx + 1))
+                    names.append(f"{obsname}.{idx + 1}")
         # make sure the length of names is greater than or equal to nobs
         else:
             if isinstance(names, str):
                 names = [names]
             elif not isinstance(names, list):
-                msg = (
+                raise ValueError(
                     "HeadObservation names must be a "
-                    + "string or a list of strings"
+                    "string or a list of strings"
                 )
-                raise ValueError(msg)
             if len(names) < self.nobs:
-                msg = (
-                    "a name must be specified for every valid "
-                    + "observation - {} ".format(len(names))
-                    + "names were passed but at least "
-                    + "{} names are required.".format(self.nobs)
+                raise ValueError(
+                    "a name must be specified for every valid observation "
+                    "- {} names were passed but at least "
+                    "{} names are required.".format(len(names), self.nobs)
                 )
-                raise ValueError(msg)
 
         # create time_series_data
         self.time_series_data = self._get_empty(ncells=shape[0])
         for idx in range(self.nobs):
             t = time_series_data[idx, 0]
-            kstp, kper, toffset = model.dis.get_kstp_kper_toffset(t)
+            kstp, kper, toffset = model.dis.get_kstp_kper_toffset(
+                t, use_cached_totim=True
+            )
             self.time_series_data[idx]["totim"] = t
             self.time_series_data[idx]["irefsp"] = kper
             self.time_series_data[idx]["toffset"] = toffset / tomulth
