@@ -3,7 +3,7 @@ __author__ = "emorway"
 import numpy as np
 
 from ..pakbase import Package
-from ..utils import Util2d, Util3d, Transient2d
+from ..utils import Transient2d, Util2d
 
 
 class Mt3dUzt(Package):
@@ -144,7 +144,7 @@ class Mt3dUzt(Package):
         extension="uzt",
         unitnumber=None,
         filenames=None,
-        **kwargs
+        **kwargs,
     ):
 
         # set default unit number of one is not specified
@@ -154,45 +154,26 @@ class Mt3dUzt(Package):
             unitnumber = Mt3dUzt._reservedunit()
 
         # set filenames
-        if filenames is None:
-            filenames = [None, None]
-        elif isinstance(filenames, str):
-            filenames = [filenames, None, None]
-        elif isinstance(filenames, list):
-            if len(filenames) < 2:
-                for idx in range(len(filenames), 2):
-                    filenames.append(None)
+        filenames = self._prepare_filenames(filenames, 2)
 
         if icbcuz is not None:
-            fname = filenames[1]
-            extension = "uzcobs.out"
             model.add_output_file(
                 icbcuz,
-                fname=fname,
-                extension=extension,
+                fname=filenames[1],
+                extension="uzcobs.out",
                 binflag=False,
-                package=Mt3dUzt._ftype(),
+                package=self._ftype(),
             )
         else:
             icbcuz = 0
 
-        # Fill namefile items
-        name = [Mt3dUzt._ftype()]
-        units = [unitnumber]
-        extra = [""]
-
-        # set package name
-        fname = [filenames[0]]
-
-        # Call ancestor's init to set self.parent, extension, name and unit number
-        Package.__init__(
-            self,
+        # call base package constructor
+        super().__init__(
             model,
             extension=extension,
-            name=name,
-            unit_number=units,
-            extra=extra,
-            filenames=fname,
+            name=self._ftype(),
+            unit_number=unitnumber,
+            filenames=filenames[0],
         )
 
         # Set dimensions
@@ -243,13 +224,13 @@ class Mt3dUzt(Package):
             if ncomp > 1:
                 for icomp in range(2, ncomp + 1):
                     val = 0.0
-                    name = "cuzinf" + str(icomp)
+                    name = f"cuzinf{icomp}"
                     if name in list(kwargs.keys()):
                         val = kwargs.pop(name)
                     else:
                         print(
-                            "UZT: setting cuzinf for component {} to zero. "
-                            "kwarg name {}".format(icomp, name)
+                            "UZT: setting cuzinf for component "
+                            f"{icomp} to zero. kwarg name {name}"
                         )
 
                     t2d = Transient2d(
@@ -276,13 +257,13 @@ class Mt3dUzt(Package):
             if ncomp > 1:
                 for icomp in range(2, ncomp + 1):
                     val = 0.0
-                    name = "cuzet" + str(icomp)
+                    name = f"cuzet{icomp}"
                     if name in list(kwargs.keys()):
                         val = kwargs.pop(name)
                     else:
                         print(
-                            "UZT: setting cuzet for component {} to zero. "
-                            "kwarg name {}".format(icomp, name)
+                            "UZT: setting cuzet for component "
+                            f"{icomp} to zero. kwarg name {name}"
                         )
 
                     t2d = Transient2d(
@@ -309,13 +290,13 @@ class Mt3dUzt(Package):
             if ncomp > 1:
                 for icomp in range(2, ncomp + 1):
                     val = 0.0
-                    name = "cgwet" + str(icomp)
+                    name = f"cgwet{icomp}"
                     if name in list(kwargs.keys()):
                         val = kwargs.pop(name)
                     else:
                         print(
-                            "UZT: setting cgwet for component {} to zero. "
-                            "kwarg name {}".format(icomp, name)
+                            "UZT: setting cgwet for component "
+                            f"{icomp} to zero. kwarg name {name}"
                         )
 
                     t2d = Transient2d(
@@ -345,13 +326,11 @@ class Mt3dUzt(Package):
         f_uzt = open(self.fn_path, "w")
 
         # Write header
-        f_uzt.write("#{0:s}\n".format(self.heading1))
+        f_uzt.write(f"#{self.heading1}\n")
 
         # Item 2
         f_uzt.write(
-            "{0:10d}{1:10d}                    #ICBCUZ, IET\n".format(
-                self.icbcuz, self.iet
-            )
+            f"{self.icbcuz:10d}{self.iet:10d}                    #ICBCUZ, IET\n"
         )
 
         # Item 3
@@ -375,9 +354,7 @@ class Mt3dUzt(Package):
                     if incuzinf == 1:
                         break
                 f_uzt.write(
-                    "{0:10d}          # INCUZINF - SP {1:5d}\n".format(
-                        incuzinf, kper + 1
-                    )
+                    f"{incuzinf:10d}          # INCUZINF - SP {kper + 1:5d}\n"
                 )
                 if incuzinf == 1:
                     for t2d in self.cuzinf:
@@ -396,9 +373,7 @@ class Mt3dUzt(Package):
                         if incuzet == 1:
                             break
                     f_uzt.write(
-                        "{0:10d}          # INCUZET - SP {1:5d}\n".format(
-                            incuzet, kper + 1
-                        )
+                        f"{incuzet:10d}          # INCUZET - SP {kper + 1:5d}\n"
                     )
                     if incuzet == 1:
                         for t2d in self.cuzet:
@@ -416,9 +391,7 @@ class Mt3dUzt(Package):
                         if incgwet == 1:
                             break
                     f_uzt.write(
-                        "{0:10d}          # INCGWET - SP {1:5d}\n".format(
-                            incgwet, kper + 1
-                        )
+                        f"{incgwet:10d}          # INCGWET - SP {kper + 1:5d}\n"
                     )
                     if incgwet == 1:
                         for t2d in self.cgwet:
@@ -501,7 +474,7 @@ class Mt3dUzt(Package):
         while line[0:1] == "#":
             i = 1
             if model.verbose:
-                print("   Comment Line " + str(i) + ": ".format(line.strip()))
+                print(f"   Comment Line {i}: {line.strip()}")
                 i += 1
             line = f.readline()
 
@@ -531,7 +504,7 @@ class Mt3dUzt(Package):
         cuzinf = {0: t2d}
         if ncomp > 1:
             for icomp in range(2, ncomp + 1):
-                name = "cuzinf" + str(icomp)
+                name = f"cuzinf{icomp}"
                 t2d = Transient2d(
                     model, (nrow, ncol), np.float32, 0.0, name=name, locat=0
                 )
@@ -546,7 +519,7 @@ class Mt3dUzt(Package):
             cuzet = {0: t2d}
             if ncomp > 1:
                 for icomp in range(2, ncomp + 1):
-                    name = "cuzet" + str(icomp)
+                    name = f"cuzet{icomp}"
                     t2d = Transient2d(
                         model,
                         (nrow, ncol),
@@ -565,7 +538,7 @@ class Mt3dUzt(Package):
             cgwet = {0: t2d}
             if ncomp > 1:
                 for icomp in range(2, ncomp + 1):
-                    name = "cgwet" + str(icomp)
+                    name = f"cgwet{icomp}"
                     t2d = Transient2d(
                         model,
                         (nrow, ncol),
@@ -583,7 +556,7 @@ class Mt3dUzt(Package):
         for iper in range(nper):
 
             if model.verbose:
-                print("   loading UZT data for kper {0:5d}".format(iper + 1))
+                print(f"   loading UZT data for kper {iper + 1:5d}")
 
             # Item 4 (INCUZINF)
             line = f.readline()
@@ -593,10 +566,7 @@ class Mt3dUzt(Package):
             # Item 5 (CUZINF)
             if incuzinf >= 0:
                 if model.verbose:
-                    print(
-                        "   Reading CUZINF array for kper "
-                        "{0:5d}".format(iper + 1)
-                    )
+                    print(f"   Reading CUZINF array for kper {iper + 1:5d}")
                 t = Util2d.load(
                     f, model, (nrow, ncol), np.float32, "cuzinf", ext_unit_dict
                 )
@@ -605,9 +575,9 @@ class Mt3dUzt(Package):
                 # Load each multispecies array
                 if ncomp > 1:
                     for icomp in range(2, ncomp + 1):
-                        name = "cuzinf" + str(icomp)
+                        name = f"cuzinf{icomp}"
                         if model.verbose:
-                            print("   loading {}...".format(name))
+                            print(f"   loading {name}...")
                         t = Util2d.load(
                             f,
                             model,
@@ -634,9 +604,8 @@ class Mt3dUzt(Package):
             elif incuzinf < 0 and iper > 0:
                 if model.verbose:
                     print(
-                        "   Reusing CUZINF array from kper "
-                        "{0:5d}".format(iper) + " in kper "
-                        "{0:5d}".format(iper + 1)
+                        f"   Reusing CUZINF array from kper {iper:5d}"
+                        f" in kper {iper + 1:5d}"
                     )
 
             if iet != 0:
@@ -648,10 +617,7 @@ class Mt3dUzt(Package):
                 # Item 7 (CUZET)
                 if incuzet >= 0:
                     if model.verbose:
-                        print(
-                            "   Reading CUZET array for kper "
-                            "{0:5d}".format(iper + 1)
-                        )
+                        print(f"   Reading CUZET array for kper {iper + 1:5d}")
                     t = Util2d.load(
                         f,
                         model,
@@ -665,9 +631,9 @@ class Mt3dUzt(Package):
                     # Load each multispecies array
                     if ncomp > 1:
                         for icomp in range(2, ncomp + 1):
-                            name = "cuzet" + str(icomp)
+                            name = f"cuzet{icomp}"
                             if model.verbose:
-                                print("   loading {}".format(name))
+                                print(f"   loading {name}")
                             t = Util2d.load(
                                 f,
                                 model,
@@ -693,9 +659,8 @@ class Mt3dUzt(Package):
                 else:
                     if model.verbose:
                         print(
-                            "   Reusing CUZET array from kper "
-                            "{0:5d}".format(iper) + " in kper "
-                            "{0:5d}".format(iper + 1)
+                            f"   Reusing CUZET array from kper {iper:5d}"
+                            f" in kper {iper + 1:5d}"
                         )
 
                 # Item 8 (INCGWET)
@@ -706,10 +671,7 @@ class Mt3dUzt(Package):
                 # Item 9 (CGWET)
                 if model.verbose:
                     if incuzet >= 0:
-                        print(
-                            "   Reading CGWET array for kper "
-                            "{0:5d}".format(iper + 1)
-                        )
+                        print(f"   Reading CGWET array for kper {iper + 1:5d}")
                     t = Util2d.load(
                         f,
                         model,
@@ -723,9 +685,9 @@ class Mt3dUzt(Package):
                     # Load each multispecies array
                     if ncomp > 1:
                         for icomp in range(2, ncomp + 1):
-                            name = "cgwet" + str(icomp)
+                            name = f"cgwet{icomp}"
                             if model.verbose:
-                                print("   loading {}...".format(name))
+                                print(f"   loading {name}...")
                             t = Util2d.load(
                                 f,
                                 model,
@@ -752,9 +714,8 @@ class Mt3dUzt(Package):
                 elif incgwet < 0 and iper > 0:
                     if model.verbose:
                         print(
-                            "   Reusing CGWET array from kper "
-                            "{0:5d}".format(iper) + " in kper "
-                            "{0:5d}".format(iper + 1)
+                            f"   Reusing CGWET array from kper {iper:5d}"
+                            f" in kper {iper + 1:5d}"
                         )
 
         if openfile:
@@ -783,7 +744,7 @@ class Mt3dUzt(Package):
             cgwet=cgwet,
             unitnumber=unitnumber,
             filenames=filenames,
-            **kwargs
+            **kwargs,
         )
 
     @staticmethod
