@@ -4,24 +4,14 @@ using ModelMap and ModelCrossSection. Functions for plotting
 shapefiles are also included.
 
 """
-from __future__ import print_function
 import os
-import sys
-import math
-import numpy as np
 import warnings
-from ..utils import Util3d
-from ..datbase import DataType, DataInterface
 
-try:
-    import shapefile
-except ImportError:
-    shapefile = None
+import matplotlib.pyplot as plt
+import numpy as np
 
-try:
-    import matplotlib.pyplot as plt
-except ImportError:
-    plt = None
+from ..datbase import DataInterface, DataType
+from ..utils import Util3d, import_optional_dependency
 
 warnings.simplefilter("ignore", RuntimeWarning)
 
@@ -39,11 +29,6 @@ bc_color_dict = {
 }
 
 
-class PlotException(Exception):
-    def __init__(self, message):
-        super().__init__(message)
-
-
 class PlotUtilities:
     """
     Class which groups a collection of plotting utilities
@@ -117,7 +102,7 @@ class PlotUtilities:
 
             model_filename_base = None
             if filename_base is not None:
-                model_filename_base = filename_base + "_" + model_name
+                model_filename_base = f"{filename_base}_{model_name}"
 
             if model.verbose:
                 print("   Plotting Model:   ", model_name)
@@ -132,7 +117,7 @@ class PlotUtilities:
                 key=defaults["key"],
                 initial_fig=ifig,
                 model_name=model_name,
-                **kwargs
+                **kwargs,
             )
 
             if isinstance(caxs, list):
@@ -408,7 +393,7 @@ class PlotUtilities:
                         fignum=fignum,
                         colorbar=colorbar,
                         modelgrid=defaults["modelgrid"],
-                        **kwargs
+                        **kwargs,
                     )
 
                     if ax is not None:
@@ -536,7 +521,7 @@ class PlotUtilities:
         filename_base=None,
         file_extension=None,
         mflay=None,
-        **kwargs
+        **kwargs,
     ):
         """
         Plot stress period boundary condition (MfList) data for a specified
@@ -656,20 +641,17 @@ class PlotUtilities:
                 filenames=filenames,
                 mflay=mflay,
                 modelgrid=modelgrid,
-                **kwargs
+                **kwargs,
             )
         else:
             arr_dict = mflist.to_array(kper, mask=True)
 
             try:
                 arr = arr_dict[key]
-            except:
-                err_msg = "Cannot find key to plot\n"
-                err_msg += "  Provided key={}\n  Available keys=".format(key)
-                for name, arr in arr_dict.items():
-                    err_msg += "{}, ".format(name)
-                err_msg += "\n"
-                raise PlotException(err_msg)
+            except KeyError:
+                err_msg = f'Cannot find key "{key}" to plot\n  Available keys='
+                err_msg += ", ".join(str(k) for k in arr_dict.keys())
+                raise KeyError(err_msg)
 
             axes = PlotUtilities._plot_array_helper(
                 arr,
@@ -678,7 +660,7 @@ class PlotUtilities:
                 filenames=filenames,
                 mflay=mflay,
                 modelgrid=modelgrid,
-                **kwargs
+                **kwargs,
             )
         return axes
 
@@ -689,7 +671,7 @@ class PlotUtilities:
         filename_base=None,
         file_extension=None,
         fignum=None,
-        **kwargs
+        **kwargs,
     ):
         """
         Plot 2-D model input data
@@ -752,7 +734,7 @@ class PlotUtilities:
             modelgrid = kwargs.pop("modelgrid")
 
         if title is None:
-            title = "{}{}".format(model_name, util2d.name)
+            title = f"{model_name}{util2d.name}"
 
         if file_extension is not None:
             fext = file_extension
@@ -761,7 +743,7 @@ class PlotUtilities:
 
         filename = None
         if filename_base is not None:
-            filename = "{}_{}.{}".format(filename_base, util2d.name, fext)
+            filename = f"{filename_base}_{util2d.name}.{fext}"
 
         axes = PlotUtilities._plot_array_helper(
             util2d.array,
@@ -770,7 +752,7 @@ class PlotUtilities:
             filenames=filename,
             fignum=fignum,
             modelgrid=modelgrid,
-            **kwargs
+            **kwargs,
         )
         return axes
 
@@ -781,7 +763,7 @@ class PlotUtilities:
         file_extension=None,
         mflay=None,
         fignum=None,
-        **kwargs
+        **kwargs,
     ):
         """
         Plot 3-D model input data
@@ -860,7 +842,7 @@ class PlotUtilities:
             name = [name] * nplottable_layers
 
         names = [
-            "{}{} layer {}".format(model_name, name[k], k + 1)
+            f"{model_name}{name[k]} layer {k + 1}"
             for k in range(nplottable_layers)
         ]
 
@@ -868,7 +850,7 @@ class PlotUtilities:
         if filename_base is not None:
             # build filenames, use local "name" variable (flopy6 adaptation)
             filenames = [
-                "{}_{}_Layer{}.{}".format(filename_base, name[k], k + 1, fext)
+                f"{filename_base}_{name[k]}_Layer{k + 1}.{fext}"
                 for k in range(nplottable_layers)
             ]
 
@@ -880,7 +862,7 @@ class PlotUtilities:
             mflay=mflay,
             fignum=fignum,
             modelgrid=modelgrid,
-            **kwargs
+            **kwargs,
         )
         return axes
 
@@ -891,7 +873,7 @@ class PlotUtilities:
         file_extension=None,
         kper=0,
         fignum=None,
-        **kwargs
+        **kwargs,
     ):
         """
         Plot transient 2-D model input data
@@ -990,7 +972,7 @@ class PlotUtilities:
             )
 
             if filename_base is not None:
-                filename = filename_base + "_{:05d}.{}".format(kper + 1, fext)
+                filename = f"{filename_base}_{kper + 1:05d}.{fext}"
             else:
                 filename = None
 
@@ -1002,7 +984,7 @@ class PlotUtilities:
                     filenames=filename,
                     fignum=fignum[idx],
                     modelgrid=modelgrid,
-                    **kwargs
+                    **kwargs,
                 )
             )
         return axes
@@ -1045,7 +1027,7 @@ class PlotUtilities:
         title = scalar.name.replace("_", "").upper()
 
         if filename_base is not None:
-            filename = filename_base + ".{}".format(fext)
+            filename = f"{filename_base}.{fext}"
         else:
             filename = None
 
@@ -1055,7 +1037,7 @@ class PlotUtilities:
             names=title,
             filenames=filename,
             modelgrid=modelgrid,
-            **kwargs
+            **kwargs,
         )
         return axes
 
@@ -1069,7 +1051,7 @@ class PlotUtilities:
         filenames=None,
         fignum=None,
         mflay=None,
-        **kwargs
+        **kwargs,
     ):
         """
         Helper method to plot array objects
@@ -1118,13 +1100,6 @@ class PlotUtilities:
             "modelgrid": None,
         }
 
-        # check that matplotlib is installed
-        if plt is None:
-            raise PlotException(
-                "Could not import matplotlib.  Must install matplotlib "
-                "in order to plot LayerFile data."
-            )
-
         for key in defaults:
             if key in kwargs:
                 defaults[key] = kwargs.pop(key)
@@ -1188,7 +1163,7 @@ class PlotUtilities:
                     plotarray,
                     masked_values=defaults["masked_values"],
                     ax=axes[idx],
-                    **kwargs
+                    **kwargs,
                 )
 
                 if defaults["colorbar"]:
@@ -1204,7 +1179,7 @@ class PlotUtilities:
                     ax=axes[idx],
                     colors=defaults["colors"],
                     levels=defaults["levels"],
-                    **kwargs
+                    **kwargs,
                 )
                 if defaults["clabel"]:
                     axes[idx].clabel(cl, fmt=defaults["fmt"], **kwargs)
@@ -1223,9 +1198,7 @@ class PlotUtilities:
             for idx, k in enumerate(range(i0, i1)):
                 fig = plt.figure(num=fignum[idx])
                 fig.savefig(filenames[idx], dpi=defaults["dpi"])
-                print(
-                    "    created...{}".format(os.path.basename(filenames[idx]))
-                )
+                print(f"    created...{os.path.basename(filenames[idx])}")
             # there will be nothing to return when done
             axes = None
             plt.close("all")
@@ -1241,7 +1214,7 @@ class PlotUtilities:
         filenames=None,
         fignum=None,
         mflay=None,
-        **kwargs
+        **kwargs,
     ):
         """
         Helper method to plot bc objects from flopy packages
@@ -1272,12 +1245,6 @@ class PlotUtilities:
 
         from .map import PlotMapView
 
-        if plt is None:
-            raise PlotException(
-                "Could not import matplotlib.  Must install matplotlib "
-                "in order to plot boundary condition data."
-            )
-
         defaults = {
             "figsize": None,
             "inactive": True,
@@ -1342,9 +1309,7 @@ class PlotUtilities:
                 fig = plt.figure(num=fignum[idx])
                 fig.savefig(filenames[idx], dpi=defaults["dpi"])
                 plt.close(fignum[idx])
-                print(
-                    "    created...{}".format(os.path.basename(filenames[idx]))
-                )
+                print(f"    created...{os.path.basename(filenames[idx])}")
             # there will be nothing to return when done
             axes = None
             plt.close("all")
@@ -1405,13 +1370,10 @@ class PlotUtilities:
         if names is not None:
             if not isinstance(names, list):
                 if maxlay > 1:
-                    names = [
-                        "{} layer {}".format(names, i + 1)
-                        for i in range(maxlay)
-                    ]
+                    names = [f"{names} layer {i + 1}" for i in range(maxlay)]
                 else:
                     names = [names]
-            msg = "{} /= {}: {}".format(len(names), maxlay, names)
+            msg = f"{len(names)} /= {maxlay}: {names}"
             assert len(names) == maxlay, msg
         return names
 
@@ -1441,7 +1403,7 @@ class PlotUtilities:
         if fignum is not None:
             if not isinstance(fignum, list):
                 fignum = [fignum]
-            msg = "{} /= {}".format(len(fignum), maxlay)
+            msg = f"{len(fignum)} /= {maxlay}"
             assert len(fignum) == maxlay, msg
             # check for existing figures
             f0 = fignum[0]
@@ -1505,7 +1467,7 @@ class PlotUtilities:
                     klay = k
                     if mflay is not None:
                         klay = int(mflay)
-                    title = "{} Layer {}".format("data", klay + 1)
+                    title = f"data Layer {klay + 1}"
                 ax.set_title(title)
                 axes.append(ax)
 
@@ -1625,8 +1587,6 @@ class PlotUtilities:
             Specific discharge arrays that have been interpolated to cell centers.
 
         """
-        import warnings
-
         warnings.warn(
             "centered_specific_discharge() has been deprecated and will be "
             "removed in version 3.3.5. Use "
@@ -1778,11 +1738,13 @@ class UnstructuredPlotUtilities:
                         cells.append(cell)
                         cell_vertex_ix.append(cvert_ix)
 
-            # find interesection vertices
+            # find intersection vertices
             numa = (x4 - x3) * (y1 - y3) - (y4 - y3) * (x1 - x3)
             numb = (x2 - x1) * (y1 - y3) - (y2 - y1) * (x1 - x3)
             denom = (y4 - y3) * (x2 - x1) - (x4 - x3) * (y2 - y1)
-            ua = numa / denom
+            ua = np.zeros(denom.shape, dtype=denom.dtype)
+            idx = np.where(denom != 0.0)
+            ua[idx] = numa[idx] / denom[idx]
             # ub = numb / denom
             del numa
             del numb
@@ -1952,7 +1914,7 @@ class SwiConcentration:
             try:
                 dis = model.get_package("DIS")
             except:
-                sys.stdout.write("Error: DIS package not available.\n")
+                print("Error: DIS package not available.")
             self.__botm = np.zeros((dis.nlay + 1, dis.nrow, dis.ncol), float)
             self.__botm[0, :, :] = dis.top.array
             self.__botm[1:, :, :] = dis.botm.array
@@ -1962,7 +1924,7 @@ class SwiConcentration:
                 self.__istrat = swi.istrat
                 self.__nsrf = swi.nsrf
             except (AttributeError, ValueError):
-                sys.stdout.write("Error: SWI2 package not available...\n")
+                print("Error: SWI2 package not available...")
         self.__nlay = self.__botm.shape[0] - 1
         self.__nrow = self.__botm[0, :, :].shape[0]
         self.__ncol = self.__botm[0, :, :].shape[1]
@@ -2038,9 +2000,7 @@ def shapefile_extents(shp):
     >>> extent = flopy.plot.plotutil.shapefile_extents(fshp)
 
     """
-    if shapefile is None:
-        s = "Could not import shapefile.  Must install pyshp in order to plot shapefiles."
-        raise PlotException(s)
+    shapefile = import_optional_dependency("shapefile")
 
     sf = shapefile.Reader(shp)
     shapes = sf.shapes()
@@ -2080,9 +2040,7 @@ def shapefile_get_vertices(shp):
     >>> lines = flopy.plot.plotutil.shapefile_get_vertices(fshp)
 
     """
-    if shapefile is None:
-        s = "Could not import shapefile.  Must install pyshp in order to plot shapefiles."
-        raise PlotException(s)
+    shapefile = import_optional_dependency("shapefile")
 
     sf = shapefile.Reader(shp)
     shapes = sf.shapes()
@@ -2131,22 +2089,12 @@ def shapefile_to_patch_collection(shp, radius=500.0, idx=None):
             Patch collection of shapes in the shapefile
 
     """
-    if shapefile is None:
-        raise PlotException(
-            "Could not import shapefile.  Must install pyshp "
-            "in order to plot shapefiles."
-        )
-    if plt is None:
-        raise ImportError(
-            "matplotlib must be installed to "
-            "use shapefile_to_patch_collection()"
-        )
-    else:
-        from matplotlib.patches import Polygon, Circle, PathPatch
-        import matplotlib.path as MPath
-        from matplotlib.collections import PatchCollection
-        from ..utils.geospatial_utils import GeoSpatialCollection
-        from ..utils.geometry import point_in_polygon
+    import matplotlib.path as MPath
+    from matplotlib.collections import PatchCollection
+    from matplotlib.patches import Circle, PathPatch, Polygon
+
+    from ..utils.geometry import point_in_polygon
+    from ..utils.geospatial_utils import GeoSpatialCollection
 
     geofeats = GeoSpatialCollection(shp)
     shapes = geofeats.shape
@@ -2240,7 +2188,7 @@ def plot_shapefile(
     a=None,
     masked_values=None,
     idx=None,
-    **kwargs
+    **kwargs,
 ):
     """
     Generic function for plotting a shapefile.
@@ -2278,14 +2226,6 @@ def plot_shapefile(
     --------
 
     """
-
-    if shapefile is None:
-        s = (
-            "Could not import shapefile.  Must install pyshp in "
-            "order to plot shapefiles."
-        )
-        raise PlotException(s)
-
     vmin = kwargs.pop("vmin", None)
     vmax = kwargs.pop("vmax", None)
 
@@ -2339,13 +2279,8 @@ def cvfd_to_patch_collection(verts, iverts):
         DeprecationWarning,
     )
 
-    if plt is None:
-        raise ImportError(
-            "matplotlib must be installed to use cvfd_to_patch_collection()"
-        )
-    else:
-        from matplotlib.patches import Polygon
-        from matplotlib.collections import PatchCollection
+    from matplotlib.collections import PatchCollection
+    from matplotlib.patches import Polygon
 
     ptchs = []
     for ivertlist in iverts:
@@ -2371,7 +2306,7 @@ def plot_cvfd(
     facecolor="scaled",
     a=None,
     masked_values=None,
-    **kwargs
+    **kwargs,
 ):
     """
     Generic function for plotting a control volume finite difference grid of
@@ -2415,9 +2350,6 @@ def plot_cvfd(
         "Use PlotMapView for plotting",
         DeprecationWarning,
     )
-    if plt is None:
-        err_msg = "matplotlib must be installed to use plot_cvfd()"
-        raise ImportError(err_msg)
 
     if "vmin" in kwargs:
         vmin = kwargs.pop("vmin")
@@ -2527,13 +2459,7 @@ def _set_coord_info(mg, xul, yul, xll, yll, rotation):
     -------
     mg : fp.discretization.Grid object
     """
-    import warnings
-
     if xul is not None and yul is not None:
-        warnings.warn(
-            "xul/yul have been deprecated. Use xll/yll instead.",
-            DeprecationWarning,
-        )
         if rotation is not None:
             mg._angrot = rotation
 
@@ -2566,8 +2492,7 @@ def _depreciated_dis_handler(modelgrid, dis):
 
     """
     # creates a new modelgrid instance with the dis information
-    from ..discretization import StructuredGrid, VertexGrid, UnstructuredGrid
-    import warnings
+    from ..discretization import StructuredGrid, UnstructuredGrid, VertexGrid
 
     warnings.warn(
         "the dis parameter has been depreciated and will be removed in "
@@ -2642,7 +2567,7 @@ def advanced_package_bc_helper(pkg, modelgrid, kper):
             idx = np.array(idx)
     else:
         raise NotImplementedError(
-            "Pkg {} not implemented for bc plotting".format(pkg.package_type)
+            f"Pkg {pkg.package_type} not implemented for bc plotting"
         )
     return idx
 
@@ -2753,9 +2678,7 @@ def intersect_modpath_with_crosssection(
     nppts = {}
 
     for cell, verts in projpts.items():
-        tcell = cell
-        while tcell >= ncpl:
-            tcell -= ncpl
+        tcell = cell % ncpl
         zmin = np.min(np.array(verts)[:, 1])
         zmax = np.max(np.array(verts)[:, 1])
         nmin = np.min(v_norm[tcell])
@@ -2879,7 +2802,7 @@ def reproject_modpath_to_crosssection(
                 rec[xp] = x
                 rec[yp] = y
                 pid = rec["particleid"][0]
-                pline = list(zip(rec[proj], rec[zp]))
+                pline = list(zip(rec[proj], rec[zp], rec["time"]))
                 if pid not in ptdict:
                     ptdict[pid] = pline
                 else:
@@ -2897,7 +2820,7 @@ def reproject_modpath_to_crosssection(
                 rec[xp] = x
                 rec[yp] = y
                 pid = rec["particleid"][0]
-                pline = list(zip(rec[proj], rec[zp]))
+                pline = list(zip(rec[proj], rec[zp], rec["time"]))
                 if pid not in ptdict:
                     ptdict[pid] = pline
                 else:
